#  Lawson Labs, Inc.
#  Open Source Python 2.7 code for Model 201 and EMF16
#  Feb 2, 2017
#  www.lawsonlabs.com
#  lawsnlab@lawsonlabs.com
#  justScanrev1.py
#
#  This demo loads the Drivr201.cfg file info, starts a scan,
#  and writes a data file.
#
#  Other resources:
#  DRIVR201.BAS original, well remarked source code
#               with simillar variables and parallel function
#
#  NT_m201 C application with parallel function using a dll
#  The Model 201 Manual documents hardware, including the command structure
#
#  We welcome questions and comments and even bug reports


import serial
import time
import msvcrt
import curses

#import logging
#LOG_FILENAME = 'debug.txt'
#$logging.basicConfig(filename=LOG_FILENAME,level=logging.DEBUG)
#logging.debug('start of debug\n')

stdscr = curses.initscr()
curses.cbreak()
stdscr.keypad(1)
curses.echo()
curses.curs_set(0)

global CHECKSUM, rawcount, ser, version, SCALE16, SCALE24
global inbyte, rawcount, SCALE, BU, WORDCNT, STBY, lastin, lastout
global MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG
global EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL, SINGLECHAN
global SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE, CHANR
global cnt, BAUDR, nums, NUMCCS, FTIME, Pof, adcount, WAITIME, nc, stypestr

STBY=0
CHECKSUM = 0
cnt = 0
rawcount=0
lastin="   "
lastout="   "
chks= "        "
st=""
nc = [0,0,0,0,0,0]
stypestr = [" Single Channel  ", "      Normal     ", "Self-Calibration"]

def getchar(timeout):   # waits for 1 character or timeout
    global CHECKSUM
    xx= time.time() + timeout
    while (time.time() < xx):
        cnt= ser.inWaiting()
        if (cnt > 0):
            inbyte = ser.read()
            CHECKSUM = (CHECKSUM + ord(inbyte)) % 256
            return inbyte         
    return -1

def fileread(fline, f):      #read next value from a comma-delimited file
    global cnt               # f is flag  1= return an integer value
    cf = fline.find(",", cnt)
    st= fline[cnt:cf]
    if f:
        st= int(st)
    cnt = cf + 1
    return st

def sendpacket(B1, B2):
    ser.write(chr(B1))
    ser.write(chr(B2))
    ser.write(chr((B1 + B2) % 256))

def sendgetdatacmd (B1, B2):
    global inbyte
    sendpacket(B1, B2)
    inbyte= getchar(1)
    if ord(inbyte) <> B1:
        print  ord(inbyte), B1, "command token echo error "

def getconver():    #  get an A/D conversion, calculate count and scale in volts
    inbyte = getchar(WAITIME)
    if inbyte <> -1:
        rawcount = ord(inbyte)
    else:
        rawcount = 0
        print "getconver timeout"
    inbyte = getchar(1)
    if inbyte <> -1:
        rawcount = rawcount + ord(inbyte) * 256
    else:
        print "getconver timeout1"
    if WL:
        inbyte = getchar(1)  
        rawcount = rawcount + ord(inbyte) * 65536
    VOLTS = (rawcount * SCALE)
    if BU == 0:
        VOLTS = VOLTS * 2 - 5
    return (VOLTS, rawcount)

def   syscal():            # perform system calibration
      global GAIN
      ORGAIN = GAIN
      if GAIN <> 1:
        GAIN = 1
        warmstart()
      #  IF gain not equal to 1, save original gain AND calibrate with GAIN 1
      CC = 7 * 16 
      DOCAL(CC, "O")
      CC = 6 * 16 
      DOCAL(CC, "F")
      if GAIN <> ORGAIN:
        GAIN = ORGAIN
        warmstart()
        CC = 7 * 16 
        DOCAL(CC, "O")
 

def   DOCAL(CC, A):   #  perform individual calibration on channel selected
      B2 = CC
      if A == "O":
          B1 = 0x82
      if A == "F":
          B1 = 0x83
      sendgetdatacmd (B1, B2)
      nums = range (0, WORDCNT)
      for i in nums:
          inbyte= getchar((WAITIME + FTIME)*2)   
          
def checksum():       #do get checkum command and test result
      global CHECKSUM
      sendgetdatacmd (0x87, 1)
      inbyte= getchar(1)
      if CHECKSUM <> ((ord(inbyte) * 2) % 256):
          print " M201 checksum (HEX) ", ord(inbyte), " LOCAL checksum (HEX) ", CHECKSUM
          chks = "Chksum NG"
      else:
          chks= "Chksum OK"
      CHECKSUM = 0
      return chks

def readconfig():
    global MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG
    global EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL, SINGLECHAN
    global SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE, SCALE16, SCALE24
    global cnt,BAUDR, nums, NUMCCS, WORDCNT, SCALE, FTIME, Pof, adcount, CHANR
    global CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5, WAITIME, MAXBITS

    cfg=open ("drivr201.cfg", "r")
    fline = cfg.readline()

    cnt=0
    MODEREGHI = fileread(fline, 1)
    MODEREGMID = fileread(fline, 1)
    MODEREGLO = fileread(fline, 1)
    CHANNEL = fileread(fline, 1)
    FILTER = fileread(fline, 1)
    LOCAVG = fileread(fline, 1)
    print MODEREGHI, MODEREGMID, MODEREGLO, CHANNEL, FILTER, LOCAVG

    fline = cfg.readline()
    cnt=0
    EXTCODE = fileread(fline, 1)
    BAUD = fileread(fline, 1)
    COM = fileread(fline, 0)
    COM= COM[1:len(COM)-2]
    INPBUF = fileread(fline, 1)
    NUMAVG = fileread(fline, 1)
    BU= fileread(fline, 1)
    GAIN = fileread(fline, 1)
    RATE = fileread(fline, 0)
    WL = fileread(fline,1)
    RATE=float(RATE)
    GAIN= int(GAIN)
    print EXTCODE, BAUD, COM, INPBUF, NUMAVG, BU, GAIN, RATE, WL

    fline = cfg.readline()
    cnt=0
    CHAN0 = fileread(fline, 1)
    CHAN1 = fileread(fline, 1)
    CHAN2 = fileread(fline, 1)
    CHAN3 = fileread(fline, 1)
    CHAN4 = fileread(fline, 1)
    CHAN5 = fileread(fline, 1)
    print CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5
    CHANR = [CHAN0, CHAN1, CHAN2, CHAN3, CHAN4, CHAN5]

    fline = cfg.readline()
    cnt=0
    SINGLECHAN = fileread(fline, 1)
    print SINGLECHAN

    fline = cfg.readline()
    cnt=0
    SCAN = fileread(fline, 1)
    SCAN1 = fileread(fline, 1)
    SCAN2 = fileread(fline, 1)
    SCANINTERVAL = fileread(fline, 0)
    STYPE = fileread(fline, 1)
    RESERVE = fileread(fline, 0)
    SCANINTERVAL = float(SCANINTERVAL)
    print SCAN, SCAN1, SCAN2, SCANINTERVAL, STYPE, RESERVE

    cfg.close()
    
    BAUDR =[9600, 4800, 2400, 1200, 600, 300]
    nums = range(0,6)
    NUMCCS = 0
    for N in nums:
        CC = CHANR[N] % 16  - int(CHANR[N] / 16) + 1
        NUMCCS = NUMCCS + CC
        nc[N] =CC                  # subchannels on this A/D channel  
    WAITIME = (NUMAVG / RATE) + BAUD / 10 + 2 
    
    SCALE24 = float(5.0 / 2**24)   # one count's worth for 5 volt range
    SCALE16 = float(5.0 / 2**16)   # one count's worth for 16 bit mode
    if WL == 0:
        WORDCNT = 2
        SCALE= SCALE16
        MAXBITS=16
    else:
        WORDCNT = 3
        SCALE= SCALE24
        MAXBITS=24
    if LOCAVG == 0:
        LOCAVG = 1
    if FILTER == 0:
        FTIME = float(4.77)
    if FILTER == 1:
        FTIME = float(0.61)
    if FILTER == 2:
        FTIME = float(0.036)
 
    Pof=1                        # power of 2 of remote averaging
    while (NUMAVG / 2 ** Pof) > 1:
            Pof= Pof + 1

    if STYPE==0:
        NUMCCS=1
   
    MODEREGHI = MODEREGHI & 254   # strip off standby bit
  
def signon():
    global ser, CHECKSUM
    ser = serial.Serial(baudrate=300, bytesize=8, parity="N", stopbits=1, xonxoff=0, rtscts=0, dsrdtr=0, timeout=1, writeTimeout=2 )
    ser.port= COM
    ser.setRTS(0)
    ser.setDTR(1)
    ser.open()
    if not(ser):
        erstr="port NOT opened"
        return erstr
    token = b"\x00"
    nums = range(1, 21)

    for i in nums:
        time.sleep( .2 )
        ser.write(token)
        bchar = getchar(.5)
        if (bchar == b"\x03"):
            break
        if (i > 19):
            ser.close()
            erstr="can't wake"
            return erstr
        stdscr.move(20, 15 + i * 2)
        stdscr.addstr(" .")
        stdscr.refresh()
    bchar = ser.read()  # clears buffer
    CHECKSUM = 0
    time.sleep(.20)
    token = 0x88
    ser.write(chr(token))

    time.sleep(.10)
    ser.write(chr(BAUD))
    bchar = getchar(.3)
    if (bchar<>chr(BAUD)):     
        if (bchar == -1):
             erstr = "BAUD echo timeout"
             return erstr
        erstr = "BAUD echo wrong"
        return erstr
    ser.close()
    ser = serial.Serial (bytesize=8, parity='N', stopbits=1, xonxoff=0, timeout=1, writeTimeout=2 )
    ser.port=COM
    ser.baudrate=BAUDR[BAUD] 
    ser.setRTS(0)
    ser.setDTR(1)
    ser.open()
    if(ser):
        stdscr.addstr( "  port opened at %5s BAUD" % BAUDR[BAUD])
    else:
        erstr = "port NOT opened at selected BAUD"
        return erstr
    return "OK"

def finishsignon():
    global MODEREGHI, MODEREGMID, MODEREGLO
    token = b"\x00"            # end echo test
    ser.write(token)
    B1= MODEREGHI
    B2= MODEREGMID
    sendpacket(B1, B2)
    B1= MODEREGLO
    sendpacket(B1, B2)
    B1= Pof
    B2= FILTER
    sendpacket(B1, B2)
    B2= 0                       # scan mode
    sendpacket(B1, B2)
    B1 = getchar(1)
    B2 = getchar(1)
    B3 = getchar(1)
    if (ord(B1) <> MODEREGHI) or (ord(B2) <> MODEREGMID) or (ord(B3) <> MODEREGLO):
        erstr = "no mode register echo "
        return erstr
    B1 = SCAN                    # send scan interval and channels setup
    B2 = SCAN1
    sendpacket(B1, B2)
    B1 = SCAN2
    B2 = CHANR[0]
    sendpacket(B1, B2)
    B1 = CHANR[1]
    B2 = CHANR[2]
    sendpacket(B1, B2)
    B1 = CHANR[3]
    B2 = CHANR[4]
    sendpacket(B1, B2)
    B1 = CHANR[5]
    B2 = 0
    sendpacket(B1, B2)     

    return "OK"


def scan(doLog):
    global WAITIME
    if(doLog):
        startTime=time.strftime("%m-%d-%y %H:%M:%S")
        nm=time.strftime("%m%d%y_%H_%M_%S")
        nm= nm + ".txt"
        logFile=open (nm, "w")
        logFile.write("scan#\t")
        if(STYPE == 2):
            logFile.write(" OFFSET\t FULLSCALE\t")
        nums=range(1, NUMCCS + 1)
        for N in nums: 
            logFile.write(" %7d\t" % (N - 1))
        logFile.write("\n")
    idx=0
    if STYPE == 2:
        SCANBYTES = ((NUMCCS + 2) * WORDCNT) + 2
        B1 = 0x8C
            # calibrate scan token   &H8C
    if STYPE == 1:
        SCANBYTES = (NUMCCS * WORDCNT) + 2
        B1 = 0x89
            # normal scan command token
    if STYPE == 0:
        SCANBYTES = WORDCNT
            # singlechan has no start and end markers
        sendpacket (1, SINGLECHAN)
        time.sleep( .1)   
        B1 = 0x8B
            # single-channel scan token  &H8B
    sendgetdatacmd (B1, 0)
    stdscr.clear()
    stdscr.move(23,5)
    stdscr.addstr( "Press E to end scan") 
    stdscr.move(0,0)
    stdscr.addstr( "scan #               started at: " + startTime) 
    stdscr.refresh()
    moreinBuff=0
    ENDSCANSENT=0
    SCANCNT = 0
    WAITIME = SCANINTERVAL * 2
    while (ENDSCANSENT ==0) or moreinBuff:
        SCANCNT = SCANCNT + 1
        if(doLog):
            logFile.write("%6d\t" % SCANCNT)     
        
        if STYPE <> 0:
           inbyte=getchar(WAITIME)
           if ord(inbyte) <> 0xF0:
               if ord(inbyte) == 0x0a:
                   print "Scan interval too short"
               else:
                   print "Scan start framing error"
               quit()
        idx = (idx % 20) + 1
        stdscr.move(idx +1, 0)
        stdscr.addstr( "%2d " % SCANCNT)
        stdscr.refresh()     
        if STYPE == 2: 
            VOLTS, rawcount = getconver()
            OFFSET=VOLTS
            VOLTS, rawcount = getconver()
            FSVOLTS = VOLTS - OFFSET        
            stdscr.addstr( "  %2.6f " % OFFSET) if OFFSET >= 0 else stdscr.addstr( " %2.6f " % OFFSET)
            stdscr.addstr( "  %2.6f " % FSVOLTS) if FSVOLTS >= 0 else stdscr.addstr( " %2.6f " % FSVOLTS)
            if(doLog): 
                logFile.write(" %2.6f\t " % OFFSET) if OFFSET >= 0 else logFile.write( "%2.6f\t " % OFFSET)
                logFile.write(" %2.6f\t" % FSVOLTS) if FSVOLTS >= 0 else logFile.write( "%2.6f\t " % FSVOLTS)
            
        nums = range(0, NUMCCS)
        for N in nums: 
            VOLTS, rawcount = getconver()
            if STYPE==2:
                VOLTS=VOLTS-OFFSET
            stdscr.addstr( "  %2.6f " % VOLTS) if VOLTS >= 0 else stdscr.addstr( " %2.6f " % VOLTS)
            if(doLog): 
                logFile.write(" %2.6f\t" % VOLTS) if VOLTS >= 0 else logFile.write("%2.6f\t" % VOLTS)
            stdscr.refresh()             
        if(doLog): 
            logFile.write("\n")
        if STYPE <> 0:            #  should be end of scan token
            inbyte=getchar(1)
            if ord(inbyte) <> 0x0F:
              print "Scan framing failure (end of packet)", ord(inbyte)
              quit()

        if msvcrt.kbhit():
            keyhit= msvcrt.getch()
            if keyhit == "E" or keyhit == "e":
                sendpacket(0x8A, 1)
                ENDSCANSENT=1
                time.sleep( .5) 
        morein = ser.inWaiting() 
        if morein > 1:
             moreinBuff = 1
        else:
             moreinBuff = 0
    inbyte= getchar(2)
   
    if ord(inbyte) == 0x8a:    
         erstr= "End scan OK"
    else:
        if ord(inbyte) == 0x0a:
            erstr= "interval too small "
        else:
            erstr= "No end scan token "
       
    print erstr
    if(doLog):
        endTime=time.strftime("%m-%d-%y %H:%M:%S")
        logFile.write("\nAveraging: %d" % NUMAVG)
        logFile.write("\nInternal Data Rate: %.2f" % RATE)
        logFile.write("\nNumber of Scans: %d" % SCANCNT)
        logFile.write("\nStarted: %s" % startTime)
        logFile.write("\nFinished: %s" % endTime)
        logFile.write("\n%s" % erstr)
        logFile.close()

print ("Start time : %s" % time.ctime())

readconfig()
signon()
finishsignon()
syscal()
scan(1)
curses.curs_set(1)

quit()



